@@ -6,7 +6,7 @@ sub _getstash { \%{"$_[0]::"} }
 use strict;
 use warnings;
 
-our $VERSION = '2.000003';
+our $VERSION = '2.000004';
 $VERSION = eval $VERSION;
 
 our %INFO;
@@ -19,7 +19,6 @@ our @ON_ROLE_CREATE;
 
 BEGIN {
   *_WORK_AROUND_BROKEN_MODULE_STATE = "$]" < 5.009 ? sub(){1} : sub(){0};
-  *_MRO_MODULE = "$]" < 5.010 ? sub(){"MRO/Compat.pm"} : sub(){"mro.pm"};
 }
 
 sub croak {
@@ -162,8 +161,6 @@ sub create_class_with_roles {
     croak "${role} is not a Role::Tiny" unless $me->is_role($role);
   }
 
-  require(_MRO_MODULE);
-
   my $composite_info = $me->_composite_info_for(@roles);
   my %conflicts = %{$composite_info->{conflicts}};
   if (keys %conflicts) {
@@ -171,7 +168,7 @@ sub create_class_with_roles {
       join "\n",
         map {
           "Method name conflict for '$_' between roles "
-          ."'".join(' and ', sort values %{$conflicts{$_}})."'"
+          ."'".join("' and '", sort values %{$conflicts{$_}})."'"
           .", cannot apply these simultaneously to an object."
         } keys %conflicts;
     croak $fail;
@@ -345,8 +342,9 @@ sub _concrete_methods_of {
   my $not_methods = { reverse %{$info->{not_methods}||{}} };
   $info->{methods} ||= +{
     # grab all code entries that aren't in the not_methods list
-    map {
-      my $code = *{$stash->{$_}}{CODE};
+    map {;
+      no strict 'refs';
+      my $code = exists &{"${role}::$_"} ? \&{"${role}::$_"} : undef;
       ( ! $code or exists $not_methods->{$code} ) ? () : ($_ => $code)
     } grep !ref($stash->{$_}), keys %$stash
   };
@@ -456,8 +454,7 @@ sub _install_does {
 
 sub does_role {
   my ($proto, $role) = @_;
-  require(_MRO_MODULE);
-  foreach my $class (@{mro::get_linear_isa(ref($proto)||$proto)}) {
+  foreach my $class (@{_get_isa(ref($proto)||$proto)}) {
     return 1 if exists $APPLIED_TO{$class}{$role};
   }
   return 0;
@@ -468,6 +465,30 @@ sub is_role {
   return !!($INFO{$role} && ($INFO{$role}{is_role} || $INFO{$role}{not_methods}));
 }
 
+sub _get_isa ($;$) {
+  no warnings 'redefine';
+  if ("$]" >= 5.010) {
+    require mro;
+    *__get_isa = \&mro::get_linear_isa;
+  }
+  else {
+    *__get_isa = \&__get_linear_isa_dfs;
+  }
+  goto &__get_isa;
+}
+
+sub _get_linear_isa_dfs ($;$) {
+  my @isa = shift;
+  my @lin;
+  my %seen;
+  while (my $class = pop @isa) {
+    push @lin, $class;
+    no strict 'refs';
+    push @isa, grep !$seen{$_}++, reverse @{$class.'::ISA'};
+  }
+  return \@lin;
+}
+
 1;
 __END__
 
